import argparse
import io
import sys

from . import snappy_formats as formats
from .snappy import UncompressError


def cmdline_main():
    """This method is what is run when invoking snappy via the commandline.
    Try python -m snappy --help
    """
    stdin = sys.stdin
    if hasattr(sys.stdin, "buffer"):
        stdin = sys.stdin.buffer
    stdout = sys.stdout
    if hasattr(sys.stdout, "buffer"):
        stdout = sys.stdout.buffer

    parser = argparse.ArgumentParser(
        description="Compress or decompress snappy archive"
    )

    group = parser.add_mutually_exclusive_group(required=True)

    group.add_argument(
        '-c',
        dest='compress',
        action='store_true',
        help='Compress'
    )
    group.add_argument(
        '-d',
        dest='decompress',
        action='store_true',
        help='Decompress'
    )

    parser.add_argument(
        '-t',
        dest='target_format',
        default=formats.DEFAULT_FORMAT,
        choices=formats.ALL_SUPPORTED_FORMATS,
        help=(
            'Target format, default is "{}"'.format(formats.DEFAULT_FORMAT)
        )
    )

    parser.add_argument(
        'infile',
        nargs='?',
        type=argparse.FileType(mode='rb'),
        default=stdin,
        help="Input file (or stdin)"
    )
    parser.add_argument(
        'outfile',
        nargs='?',
        type=argparse.FileType(mode='wb'),
        default=stdout,
        help="Output file (or stdout)"
    )

    args = parser.parse_args()

    # workaround for https://bugs.python.org/issue14156
    if isinstance(args.infile, io.TextIOWrapper):
        args.infile = stdin
    if isinstance(args.outfile, io.TextIOWrapper):
        args.outfile = stdout

    additional_args = {}
    if args.compress:
        method = formats.get_compress_function(args.target_format)
    else:
        try:
            method, read_chunk = formats.get_decompress_function(
                args.target_format,
                args.infile
            )
        except UncompressError as err:
            sys.exit("Failed to get decompress function: {}".format(err))
        additional_args['start_chunk'] = read_chunk

    method(args.infile, args.outfile, **additional_args)


if __name__ == "__main__":
    cmdline_main()



from _snappy_cffi import ffi, lib

class UncompressError(Exception):
    pass

class SnappyBufferSmallError(Exception):
    pass


def prepare(data):
    _out_data = None
    _out_size = None

    _out_data = ffi.new('char[]', data)
    _out_size = ffi.cast('size_t', len(data))

    return (_out_data, _out_size)


def compress(data):
    if isinstance(data, unicode):
        data = data.encode('utf-8')

    _input_data, _input_size = prepare(data)

    max_compressed = lib.snappy_max_compressed_length(_input_size)

    _out_data = ffi.new('char[]', max_compressed)
    _out_size = ffi.new('size_t*', max_compressed)

    rc = lib.snappy_compress(_input_data, _input_size, _out_data, _out_size)

    if rc != lib.SNAPPY_OK:
        raise SnappyBufferSmallError()

    value = ffi.buffer(ffi.cast('char*', _out_data), _out_size[0])

    return value[:]


def uncompress(data):
    _out_data, _out_size = prepare(data)

    result = ffi.new('size_t*', 0)

    rc = lib.snappy_validate_compressed_buffer(_out_data, _out_size)

    if not rc == lib.SNAPPY_OK:
        raise UncompressError()

    rc = lib.snappy_uncompressed_length(_out_data,
                                      _out_size,
                                      result)

    if not rc == lib.SNAPPY_OK:
        raise UncompressError()

    _uncompressed_data = ffi.new('char[]', result[0])

    rc = lib.snappy_uncompress(_out_data, _out_size, _uncompressed_data, result)

    if rc != lib.SNAPPY_OK:
        raise UncompressError()

    buf =  ffi.buffer(ffi.cast('char*', _uncompressed_data), result[0])

    return buf[:]


def isValidCompressed(data):
    if isinstance(data, unicode):
        data = data.encode('utf-8')

    _out_data, _out_size= prepare(data)

    rc = lib.snappy_validate_compressed_buffer(_out_data, _out_size)

    return rc == lib.SNAPPY_OK

decompress = uncompress

def _crc32c(data):
    c_data = ffi.new('char[]', data)
    size = ffi.cast('int', len(data))
    return int(lib._crc32c(c_data, size))
